VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "FileFinderCore"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' Core Class for Finding Files in Windows 
'
' 29 September 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' Variables

Private hFind As Long
Private FindWhat As String
Private FindData As WIN32_FIND_DATA


'------------------------------------------------------------------
' API Declarations for finding Files

Private Const MAX_PATH = 260
Private Const INVALID_HANDLE_VALUE = -1

  ' Some aren't the same as the vb versions.
Public Enum FileAttributes
  FILE_ATTRIBUTE_ARCHIVE = &H20
  FILE_ATTRIBUTE_COMPRESSED = &H800
  FILE_ATTRIBUTE_DIRECTORY = &H10
  FILE_ATTRIBUTE_HIDDEN = &H2
  FILE_ATTRIBUTE_NORMAL = &H80
  FILE_ATTRIBUTE_READONLY = &H1
  FILE_ATTRIBUTE_SYSTEM = &H4
  FILE_ATTRIBUTE_TEMPORARY = &H100
End Enum

'Private Type SYSTEMTIME
'  wYear As Integer
'  wMonth As Integer
'  wDayOfWeek As Integer
'  wDay As Integer
'  wHour As Integer
'  wMinute As Integer
'  wSecond As Integer
'  wMilliseconds As Integer
'End Type

Private Type FILETIME
  dwLowDateTime As Long
  dwHighDateTime As Long
End Type

Private Type WIN32_FIND_DATA
  dwFileAttributes As FileAttributes
  ftCreationTime As FILETIME
  ftLastAccessTime As FILETIME
  ftLastWriteTime As FILETIME
  nFileSizeHigh As Long
  nFileSizeLow As Long
  dwReserved0 As Long
  dwReserved1 As Long
  cFileName As String * MAX_PATH
  cAlternate As String * 14
End Type

Private Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindNextFile Lib "kernel32" Alias "FindNextFileA" (ByVal hFind As Long, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindClose Lib "kernel32" (ByVal hFind As Long) As Long
'Private Declare Function FileTimeToLocalFileTime Lib "kernel32" (lpFileTime As FILETIME, lpLocalFileTime As FILETIME) As Long
'Private Declare Function FileTimeToSystemTime Lib "kernel32" (lpFileTime As FILETIME, lpSystemTime As SYSTEMTIME) As Long


'------------------------------------------------------------------
' Class Terminates: Destroy Handle

Private Sub Class_Terminate()
  If hFind > 0 Then
    On Error Resume Next
    Call Me.FindEnd
  End If
End Sub


'------------------------------------------------------------------
' Handle

Public Property Get hFindFile() As Long
  hFindFile = hFind
End Property


'------------------------------------------------------------------
' Find What?

Public Property Get FindQuery() As String
  FindQuery = FindWhat
End Property

Public Property Let FindQuery(NewValue As String)
  If hFind <> 0 Then Err.Raise 12345, "FileFinderCore", "File search has already been initialized!"
  FindWhat = NewValue
End Property


'------------------------------------------------------------------
' The FileName found

Public Property Get FileName() As String
  If hFind = 0 Then Err.Raise 12345, "FileFinderCore", "File search has not been initialized!"
  On Error Resume Next ' Left(Text, InStr[NoMatch]) Err
  FileName = Left(FindData.cFileName, InStr(1, FindData.cFileName, Chr(0)) - 1)
End Property

Public Property Get FileShortName() As String
  If hFind = 0 Then Err.Raise 12345, "FileFinderCore", "File search has not been initialized!"
  On Error Resume Next
  FileShortName = Left(FindData.cAlternate, InStr(FindData.cAlternate, Chr(0) - 1))
End Property

Public Property Get FileAttributes() As FileAttributes
  FileAttributes = FindData.dwFileAttributes
End Property


'------------------------------------------------------------------
' Start: FindFirst

Public Sub FindFirst()
  If hFind <> 0 Then Err.Raise 12345, "FileFinderCore", "File search has already been initialized!"
  hFind = FindFirstFile(FindWhat, FindData)
  If hFind = INVALID_HANDLE_VALUE Then
    Call Me.FindEnd
    Err.Raise 53, "FileSearch", "The requested file was not found: " & FindWhat
  End If
End Sub


'------------------------------------------------------------------
' Then: FindNext

Public Function FindNext() As String
  If hFind = 0 Then Err.Raise 12345, "FileFinderCore", "File search has not been initialized!"
  
  FindData.cFileName = Space(MAX_PATH) ' Clear
  If FindNextFile(hFind, FindData) = 0 Then
    Call FindClose(hFind)
    Err.Raise 53, "FileSearch", "The next requested file was not found: " & FindWhat
  End If
End Function


'------------------------------------------------------------------
' Find End: Destroy the handle

Public Sub FindEnd()
  Dim EmptyStruct As WIN32_FIND_DATA
  Call FindClose(hFind)
  LSet FindData = EmptyStruct ' Clear at once
  hFind = 0
End Sub


'------------------------------------------------------------------
'' File Found? -> Get More info!
'
'Public Function GetCreationDate(Optional TheDate As Date, Optional TheTime As Date)
'  If FileNum = 0 Then Exit Function
'
'  If FindData.ftCreationTime.dwHighDateTime = 0 Then
'    GetCreationDate = DTextUnknown
'    Exit Function
'  End If
'
'  With GetSystemTime(FindData.ftCreationTime)
'    TheDate = DateSerial(.wYear, .wMonth, .wDay)
'    TheTime = TimeSerial(.wHour, .wMinute, .wSecond)
'  End With
'  GetCreationDate = Format(TheDate + TheTime, DFormat)
'End Function
'
'Public Function GetLastAccessDate(Optional TheDate As Date, Optional TheTime As Date)
'  If FileNum = 0 Then Exit Function
'
'  If FindData.ftLastAccessTime.dwHighDateTime = 0 Then
'    GetCreationDate = DTextUnknown
'    Exit Function
'  End If
'
'  With GetSystemTime(FindData.ftLastAccessTime)
'    TheDate = DateSerial(.wYear, .wMonth, .wDay)
'    TheTime = TimeSerial(.wHour, .wMinute, .wSecond)
'  End With
'  GetLastAccessDate = Format(TheDate + TheTime, DFormat)
'End Function
'
'Public Function GetLastWriteDate(Optional TheDate As Date, Optional TheTime As Date)
'  If FileNum = 0 Then Exit Function
'
'  If FindData.ftLastWriteTime.dwHighDateTime = 0 Then
'    GetCreationDate = DTextUnknown
'    Exit Function
'  End If
'
'  With GetSystemTime(FindData.ftLastWriteTime)
'    TheDate = DateSerial(.wYear, .wMonth, .wDay)
'    TheTime = TimeSerial(.wHour, .wMinute, .wSecond)
'  End With
'  GetLastWriteDate = Format(TheDate + TheTime, DFormat)
'End Function
'
'Private Function GetSystemTime(Time As FILETIME) As SYSTEMTIME
'  Dim LocalTime As FILETIME
'  Dim SysTime As SYSTEMTIME
'  Call FileTimeToLocalFileTime(Time, LocalTime)
'  Call FileTimeToSystemTime(LocalTime, SysTime)
'  GetSystemTime = SysTime
'End Function
'
