VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "FileFinder"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' File Finder Class
'
' 29 September 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' Class Usage:
'
' Private WithEvents FileFinder as FileFinder
'
' Sub Button_Click
'   Set FileFinder = New FileFinder
'   On Error Resume Next
'   FileFinder.Find("C:\", "*.*", False)
'   If Err.Number MsgBox Err.Description
' End Sub
'
' Sub FileFinder_Found(FileName As String, FileAttr As FileAttributes)
'   MsgBox "Found: " & FileName
' End Sub
'
'------------------------------------------------------------------



'------------------------------------------------------------------
' Events Raised from this class

Public Event Found(FileName As String, FileAttr As FileAttributes)
Public Event DirChange(DirName As String)
Public Event SearchInitialize()
Public Event SearchTerminate(Canceled As Boolean)

Private Searching As Boolean ' Abort [recursive] search loop


'------------------------------------------------------------------
' Canceling the Search

Private Sub Class_Terminate()
  Me.Cancel
End Sub

Public Sub Cancel()
  Searching = False
End Sub


'------------------------------------------------------------------
' Start the search for files

Public Sub Find(ByVal InDir As String, Pattern As String, Optional IncludeSubDirs As Boolean)
  Dim ErrNum As Long
  Dim ErrStr As String
  If Right(InDir, 1) <> "\" Then InDir = InDir & "\"
  
  If Searching Then ' Already searching
    Err.Raise 12345, "FileSearch", "FileSearch is already searching!"
    Exit Sub
  End If
  
  Searching = True
  RaiseEvent SearchInitialize
  
  On Error GoTo EndFind
  If IncludeSubDirs Then
    Call FindFilesInAllDirs(InDir, Pattern)
  Else
    Call FindFilesInThisDir(InDir, Pattern)
  End If
EndFind:
  If Err.Number Then
    ErrNum = Err.Number
    ErrStr = Err.Description
  End If
  RaiseEvent SearchTerminate(Not Searching)
  Searching = False
On Error GoTo 0
  If ErrNum Then Err.Raise ErrNum, , ErrStr
End Sub


'------------------------------------------------------------------
' Helper for Find

Private Sub FindFilesInThisDir(InDir As String, Pattern As String)
  If Not Searching Then Exit Sub
  
  Dim FileName As String
  Dim FindCore As New FileFinderCore
  FindCore.FindQuery = InDir & Pattern
  FindCore.FindFirst
  FileName = FindCore.FileName
  RaiseEvent DirChange(FileParsePath(InDir & Pattern))
  
  Do While Len(FileName)
    If Len(FileName) And Left(FileName, 1) <> "." Then RaiseEvent Found(InDir & FileName, FindCore.FileAttributes)
    
    If Not Searching Then Call FindCore.FindEnd: Exit Sub
    On Error Resume Next
      FindCore.FindNext
      FileName = FindCore.FileName
'      If Err.Number = 53 Then Exit Do
    On Error GoTo 0
  Loop
End Sub

' recursive function
Private Sub FindFilesInAllDirs(InDir As String, Pattern As String)
  If Not Searching Then Exit Sub
  
  Dim FileName As String
  Dim FindCore As New FileFinderCore
  
  FindCore.FindQuery = InDir & Pattern
  FindCore.FindFirst
  FileName = FindCore.FileName
  RaiseEvent DirChange(FileParsePath(InDir & Pattern))
  
  Do While Len(FileName)
    If Left(FileName, 1) <> "." Then  ' Avoid infinite loop (Parent-Dir to Current-Dir)
      RaiseEvent Found(InDir & FileName, FindCore.FileAttributes)
      If (FindCore.FileAttributes And FILE_ATTRIBUTE_DIRECTORY) = FILE_ATTRIBUTE_DIRECTORY Then _
         Call FindFilesInAllDirs(InDir & FileName & "\", Pattern)
    End If
    
    If Not Searching Then Call FindCore.FindEnd: Exit Sub
    On Error Resume Next
      FindCore.FindNext
      FileName = FindCore.FileName
'      If Err.Number = 53 Then Exit Do
    On Error GoTo 0
  Loop
End Sub


'------------------------------------------------------------------
' Get the path from a file

Private Function FileParsePath(ByVal FileName As String) As String
  ' Returns everything before last "\"
  Dim Pos As Long
  Do
    Pos = InStr(Pos + 1, FileName, "\")
    If Pos Then
      FileParsePath = Left(FileName, Pos)
    End If
  Loop While Pos
End Function
